#include <mpi.h>
#include <iostream>
#include <vector>
#include <cmath>
#include <cstdio>
#include "./lock-in.h"
#include "./generator-rf.h"
#include "./jade.h"
#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <stdexcept>
#include <string>

const double PI_=3.14159265358979323846;

int voxel_num = 30;
//double phase_range = 0;
double phase_range = PI_/2;
double phase_init = 0.0;

double noise_ratio = 1e8;


// B0=1.5T freq=64Mhz, period = 15.6 ns
double period_ms = 15.6/1000/1000; //ms
double total_periods = 500;
int rf_samples_per_period = 10;
// double period_ms = 10; //ms
// double total_periods = 2;
// int rf_samples_per_period = 3;

//double T2s_scale = 0.01; //ms # need to be 10ms
//double T2s_min = T2s_scale/1000.0; //
double T2s_scale = period_ms*total_periods; //ms # need to be 10ms
template<class T> inline T pow2(const T value) {return value*value;}

  
int main(int argc, char *argv[]) {
  MPI_Init(&argc, &argv);
  int rank;
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  try {
    //std::cout<<"Runing .... "<<std::endl;
    jade::SubPopulation sub_population;  // Optimizer.
    mri::GeneratorRF generator_rf;
    generator_rf.SetVoxels(voxel_num, phase_init, phase_range, T2s_scale);
    generator_rf.SetTimer(total_periods, period_ms, rf_samples_per_period);
    generator_rf.RandomizeVoxels();
    std::vector<double> signal_rf, magnitude_rf, phase_rf;
    std::vector<double> mag_sin, mag_cos;
    generator_rf.EvaluateSignal(signal_rf);
    generator_rf.EvaluateDemodulated(mag_sin,mag_cos);
    auto timesteps = generator_rf.GetTimesteps();
    mri::LockIn lock_in;
    std::vector<double> out_sin, out_cos;
    lock_in.Analyse(signal_rf, period_ms, timesteps,
                    rf_samples_per_period, out_sin, out_cos);
    for (unsigned long i = 0; i<timesteps.size(); ++i) {
      //      if (i > static_cast<unsigned long>(rf_samples_per_period)*2) 
        std::cout << timesteps[i] << "\t"
                  << signal_rf[i] << "\t"
        // << mul_sin[i] << "\t"
        // << mul_cos[i] << "\t"
                  << std::sqrt(pow2(mag_sin[i])+pow2(mag_cos[i]))
          /std::sqrt(pow2(out_sin[i])+pow2(out_cos[i])) << "\t"
                  << std::sqrt(pow2(mag_sin[i])+pow2(mag_cos[i])) << "\t"
                  << std::sqrt(pow2(out_sin[i])+pow2(out_cos[i])) << "\t"
                  << out_sin[i] << "\t"
                  << out_cos[i] << "\t"
                  << mag_sin[i] << "\t"
                  << mag_cos[i]
                  << std::endl;
    }
    //std::cout<< timesteps.size() <<std::endl;
    // for (auto value:signal_rf){
    //   std::cout << value <<std::endl;
    // }
    
  } catch( const std::invalid_argument& ia ) {
    // Will catch if  multi_layer_mie_ fails or other errors.
    std::cerr << "Invalid argument: " << ia.what() << std::endl;
    MPI_Abort(MPI_COMM_WORLD, 1);  
  }  
  MPI_Finalize();
  return 0;
}
// ********************************************************************** //
// ********************************************************************** //
// ********************************************************************** //
